import { ToggleAdditionElements } from '../../elements/table/addNewElements/shared/toggleAdditionElements';
import { RootCellElement } from '../../elements/table/addNewElements/rootCell/rootCellElement';
import { FilterInternalUtils } from '../outerTableComponents/filter/rows/filterInternalUtils';
import { AddNewRowElement } from '../../elements/table/addNewElements/row/addNewRowElement';
import { PaginationUtils } from '../outerTableComponents/pagination/paginationUtils';
import { RemoveRow } from '../insertRemoveStructure/remove/removeRow';
import { InitialDataProcessing } from '../data/initialDataProcessing';
import { InsertMatrix } from './insertMatrix';
export class UpdateAllTableData {
    static toggleAdditionalElements(at) {
        FilterInternalUtils.completeReset(at);
        // updateRowsOnNewInsert sets uses activePageNumber + 1
        if (at._pagination && at._pagination.activePageNumber !== 1)
            PaginationUtils.displayRowsForDifferentButton(at, 1);
        ToggleAdditionElements.update(at, true, AddNewRowElement.toggle);
    }
    static insertData(at, data, startRowIndex) {
        InsertMatrix.insert(at, data, startRowIndex, 0, true);
        if (startRowIndex === 0) {
            RootCellElement.convertFromRootCell(at);
        }
    }
    static changeTableData(at, data, startRowIndex, forceSync) {
        for (let i = at.data.length - 1; i >= startRowIndex; i -= 1) {
            RemoveRow.remove(at, i);
        }
        InitialDataProcessing.preProcess(at, data);
        at._isPopulatingTable = true;
        if (forceSync) {
            UpdateAllTableData.insertData(at, data, startRowIndex);
            UpdateAllTableData.toggleAdditionalElements(at);
            at._isPopulatingTable = true;
        }
        else {
            setTimeout(() => {
                // in a timeout because RemoveRow.remove contains processes inside timeouts e.g. remove column details
                UpdateAllTableData.insertData(at, data, startRowIndex);
            });
            setTimeout(() => {
                // in a timeout as at.shadowRoot.contains does not work immediately
                UpdateAllTableData.toggleAdditionalElements(at);
                at._isPopulatingTable = true;
            }, 6);
        }
    }
    static update(at, data, startRowIndex, forceSync = false) {
        if (!Array.isArray(data))
            return;
        let waitForFilterUnset = false;
        if (at._visiblityInternal.filters) {
            waitForFilterUnset = FilterInternalUtils.unsetAllFilters(at);
        }
        if (!forceSync && waitForFilterUnset) {
            // need to wait for inputElement.dispatchEvent(new Event('input')) when unsetting
            setTimeout(() => UpdateAllTableData.changeTableData(at, data, startRowIndex, forceSync), 40);
        }
        else {
            UpdateAllTableData.changeTableData(at, data, startRowIndex, forceSync);
        }
    }
}
//# sourceMappingURL=updateAllTableData.js.map